from typing import Union


def generate_ome_image_id(image_id: Union[str, int]) -> str:
    """
    Naively generates the standard OME image ID using a provided ID.

    Parameters
    ----------
    image_id: Union[str, int]
        A string or int representing the ID for an image.
        In the context of the usage of this function, this is usually used with the
        index of the scene / image.

    Returns
    -------
    ome_image_id: str
        The OME standard for image IDs.
    """
    return f"Image:{image_id}"


def generate_ome_channel_id(image_id: str, channel_id: Union[str, int]) -> str:
    """
    Naively generates the standard OME channel ID using a provided ID.

    Parameters
    ----------
    image_id: str
        An image id to pull the image specific index from.
        See: `generate_ome_image_id` for more details.
    channel_id: Union[str, int]
        A string or int representing the ID for a channel.
        In the context of the usage of this function, this is usually used with the
        index of the channel.

    Returns
    -------
    ome_channel_id: str
        The OME standard for channel IDs.

    Notes
    -----
    ImageIds are usually: "Image:0", "Image:1", or "Image:N",
    ChannelIds are usually the combination of image index + channel index --
    "Channel:0:0" for the first channel of the first image for example.
    """
    # Remove the prefix 'Image:' to get just the index
    image_index = image_id.replace("Image:", "")
    return f"Channel:{image_index}:{channel_id}"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from typing import Tuple

import numpy as np
import pytest
from bioio_base import exceptions, test_utilities

from bioio_imageio import Reader

from .conftest import LOCAL_RESOURCES_DIR


@pytest.mark.parametrize(
    "filename, set_scene, expected_shape, expected_dims_order",
    [
        ("example.bmp", "Image:0", (480, 640, 4), "YXS"),
        ("example.png", "Image:0", (800, 537, 4), "YXS"),
        ("example.jpg", "Image:0", (452, 400, 3), "YXS"),
        ("example.gif", "Image:0", (72, 268, 268, 4), "TYXS"),
        (
            "example_invalid_frame_count.mp4",
            "Image:0",
            (55, 1080, 1920, 3),
            "TYXS",
        ),
        (
            "example_valid_frame_count.mp4",
            "Image:0",
            (72, 272, 272, 3),
            "TYXS",
        ),
        pytest.param(
            "example.txt",
            None,
            None,
            None,
            marks=pytest.mark.xfail(raises=exceptions.UnsupportedFileFormatError),
        ),
        pytest.param(
            "example.png",
            "Image:1",
            None,
            None,
            marks=pytest.mark.xfail(raises=IndexError),
        ),
    ],
)
def test_reader(
    filename: str,
    set_scene: str,
    expected_shape: Tuple[int, ...],
    expected_dims_order: str,
) -> None:
    # Construct full filepath
    uri = LOCAL_RESOURCES_DIR / filename

    # Run checks
    test_utilities.run_image_file_checks(
        ImageContainer=Reader,
        image=uri,
        set_scene=set_scene,
        expected_scenes=("Image:0",),
        expected_current_scene="Image:0",
        expected_shape=expected_shape,
        expected_dtype=np.dtype(np.uint8),
        expected_dims_order=expected_dims_order,
        expected_channel_names=None,
        expected_physical_pixel_sizes=(None, None, None),
        expected_metadata_type=dict,
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from typing import List

import bioio_base.reader_metadata

###############################################################################


class ReaderMetadata(bioio_base.reader_metadata.ReaderMetadata):
    """
    Notes
    -----
    Defines metadata for the reader itself (not the image read),
    such as supported file extensions.
    """

    @staticmethod
    def get_supported_extensions() -> List[str]:
        """
        Return a list of file extensions this plugin supports reading.
        """
        # See ever growing list from imageio
        # https://imageio.readthedocs.io/en/stable/formats/index.html
        return [
            "264",
            "265",
            "3fr",
            "3g2",
            "A64",
            "IMT",
            "MCIDAS",
            "PCX",
            "SPIDER",
            "XVTHUMB",
            "a64",
            "adp",
            "amr",
            "amv",
            "apng",
            "arw",
            "asf",
            "avc",
            "avi",
            "avs",
            "avs2",
            "bay",
            "bif",
            "bmp",
            "cdg",
            "cgi",
            "cif",
            "ct",
            "dcr",
            "dib",
            "dip",
            "dng",
            "dnxhd",
            "dv",
            "dvd",
            "erf",
            "exr",
            "fff",
            "gif",
            "icb",
            "if",
            "iiq",
            "ism",
            "jif",
            "jfif",
            "jng",
            "jp2",
            "jpg",
            "mov",
            "mp4",
            "mpo",
            "msp",
            "pdf",
            "png" "ppm",
            "ps",
            "zif",
        ]

    @staticmethod
    def get_reader() -> bioio_base.reader.Reader:
        """
        Return the reader this plugin represents
        """
        from .reader import Reader

        return Reader
